@@ -34,6 +34,8 @@
 #include "dh-search.h"
 #include "dh-window.h"
 #include "dh-util.h"
+#include "dh-marshal.h"
+#include "dh-enum-types.h"
 #include "eggfindbar.h"
 #include "ige-conf.h"
 
@@ -51,12 +53,19 @@ struct _DhWindowPriv {
         GtkWidget      *vbox;
         GtkWidget      *findbar;
 
-        gint            zoom_level;
-
         GtkUIManager   *manager;
         GtkActionGroup *action_group;
+
+        DhLink         *selected_search_link;
 };
 
+enum {
+        OPEN_LINK,
+        LAST_SIGNAL
+};
+
+static gint signals[LAST_SIGNAL] = { 0 };
+
 static guint tab_accel_keys[] = {
         GDK_1, GDK_2, GDK_3, GDK_4, GDK_5,
         GDK_6, GDK_7, GDK_8, GDK_9, GDK_0
@@ -66,19 +75,19 @@ static const
 struct
 {
         gchar *name;
-        float level;
+        int    level;
 }
 zoom_levels[] =
 {
-        { N_("50%"), 0.7071067811 },
-        { N_("75%"), 0.8408964152 },
-        { N_("100%"), 1.0 },
-        { N_("125%"), 1.1892071149 },
-        { N_("150%"), 1.4142135623 },
-        { N_("175%"), 1.6817928304 },
-        { N_("200%"), 2.0 },
-        { N_("300%"), 2.8284271247 },
-        { N_("400%"), 4.0 }
+        { N_("50%"), 70 },
+        { N_("75%"), 84 },
+        { N_("100%"), 100 },
+        { N_("125%"), 119 },
+        { N_("150%"), 141 },
+        { N_("175%"), 168 },
+        { N_("200%"), 200 },
+        { N_("300%"), 283 },
+        { N_("400%"), 400 }
 };
 
 #define ZOOM_MINIMAL    (zoom_levels[0].level)
@@ -118,8 +127,9 @@ static void           window_findbar_close_cb        (GtkWidget       *widget,
                                                       DhWindow        *window);
 static GtkWidget *    window_new_tab_label           (DhWindow        *window,
                                                       const gchar     *label);
-static void           window_open_new_tab            (DhWindow        *window,
-                                                      const gchar     *location);
+static int            window_open_new_tab            (DhWindow        *window,
+                                                      const gchar     *location,
+                                                      gboolean         switch_focus);
 static WebKitWebView *window_get_active_web_view     (DhWindow        *window);
 static void           window_update_title            (DhWindow        *window,
                                                       WebKitWebView   *web_view,
@@ -154,7 +164,7 @@ window_activate_new_tab (GtkAction *action,
 
         priv = window->priv;
 
-        window_open_new_tab (window, NULL);
+        window_open_new_tab (window, NULL, TRUE);
 }
 
 static void
@@ -202,11 +212,21 @@ window_activate_copy (GtkAction *action,
                       DhWindow  *window)
 {
         GtkWidget *widget;
+        DhWindowPriv  *priv;
+
+        priv = window->priv;
 
         widget = gtk_window_get_focus (GTK_WINDOW (window));
 
         if (GTK_IS_EDITABLE (widget)) {
                 gtk_editable_copy_clipboard (GTK_EDITABLE (widget));
+        } else if (GTK_IS_TREE_VIEW (widget) &&
+                   gtk_widget_is_ancestor (widget, priv->search) &&
+                   priv->selected_search_link) {
+                GtkClipboard *clipboard;
+                clipboard = gtk_widget_get_clipboard (widget, GDK_SELECTION_CLIPBOARD);
+                gtk_clipboard_set_text (clipboard,
+                                dh_link_get_name(priv->selected_search_link), -1);
         } else {
                 WebKitWebView *web_view;
 
@@ -231,23 +251,69 @@ window_activate_find (GtkAction *action,
         webkit_web_view_set_highlight_text_matches (web_view, TRUE);
 }
 
+static int
+window_get_current_zoom_level_index (DhWindow *window)
+{
+        WebKitWebView *web_view;
+        float zoom_level;
+        int zoom_level_as_int = ZOOM_DEFAULT;
+        int i;
+
+        web_view = window_get_active_web_view (window);
+        if (web_view) {
+                g_object_get (web_view, "zoom-level", &zoom_level, NULL);
+                zoom_level_as_int = (int)(zoom_level*100);
+        }
+
+        for (i=0; zoom_levels[i].level != ZOOM_MAXIMAL; i++) {
+                if (zoom_levels[i].level == zoom_level_as_int)
+                        return i;
+        }
+        return i;
+}
+
+static void
+window_update_zoom_actions_sensitiveness (DhWindow *window)
+{
+        DhWindowPriv *priv;
+        GtkAction *zoom_in, *zoom_out, *zoom_default;
+        int zoom_level_idx;
+
+        priv = window->priv;
+        zoom_in = gtk_action_group_get_action (priv->action_group, "ZoomIn");
+        zoom_out = gtk_action_group_get_action (priv->action_group, "ZoomOut");
+        zoom_default = gtk_action_group_get_action (priv->action_group, "ZoomDefault");
+
+        zoom_level_idx = window_get_current_zoom_level_index (window);
+
+        gtk_action_set_sensitive (zoom_in,
+                                  zoom_levels[zoom_level_idx].level < ZOOM_MAXIMAL);
+        gtk_action_set_sensitive (zoom_out,
+                                  zoom_levels[zoom_level_idx].level > ZOOM_MINIMAL);
+        gtk_action_set_sensitive (zoom_default,
+                                  zoom_levels[zoom_level_idx].level != ZOOM_DEFAULT);
+}
+
 static void
 window_activate_zoom_in (GtkAction *action,
                          DhWindow  *window)
 {
         DhWindowPriv *priv;
+        int zoom_level_idx;
 
         priv = window->priv;
 
-        if (zoom_levels[priv->zoom_level].level < ZOOM_MAXIMAL) {
+        zoom_level_idx = window_get_current_zoom_level_index (window);
+        if (zoom_levels[zoom_level_idx].level < ZOOM_MAXIMAL) {
                 WebKitWebView *web_view;
 
-                priv->zoom_level++;
                 web_view = window_get_active_web_view (window);
                 g_object_set (web_view,
-                              "zoom-level", zoom_levels[priv->zoom_level].level,
+                              "zoom-level", (float)(zoom_levels[zoom_level_idx+1].level)/100,
                               NULL);
+                window_update_zoom_actions_sensitiveness (window);
         }
+
 }
 
 static void
@@ -255,17 +321,19 @@ window_activate_zoom_out (GtkAction *action,
                           DhWindow  *window)
 {
         DhWindowPriv *priv;
+        int zoom_level_idx;
 
         priv = window->priv;
 
-        if (zoom_levels[priv->zoom_level].level > ZOOM_MINIMAL) {
+        zoom_level_idx = window_get_current_zoom_level_index (window);
+        if (zoom_levels[zoom_level_idx].level > ZOOM_MINIMAL) {
                 WebKitWebView *web_view;
 
-                priv->zoom_level--;
                 web_view = window_get_active_web_view (window);
                 g_object_set (web_view,
-                              "zoom-level", zoom_levels[priv->zoom_level].level,
+                              "zoom-level", (float)(zoom_levels[zoom_level_idx-1].level)/100,
                               NULL);
+                window_update_zoom_actions_sensitiveness (window);
         }
 }
 
@@ -279,7 +347,8 @@ window_activate_zoom_default (GtkAction *action,
         priv = window->priv;
 
         web_view = window_get_active_web_view (window);
-        g_object_set (web_view, "zoom-level", ZOOM_DEFAULT, NULL);
+        g_object_set (web_view, "zoom-level", (float)(ZOOM_DEFAULT)/100, NULL);
+        window_update_zoom_actions_sensitiveness (window);
 }
 
 static void
@@ -363,8 +432,10 @@ window_activate_about (GtkAction *action,
         const gchar **documenters = NULL;
         const gchar  *translator_credits = _("translator_credits");
 
+        /* i18n: Please don't translate "Devhelp" (it's marked as translatable
+         * for transliteration only) */
         gtk_show_about_dialog (GTK_WINDOW (window),
-                               "name", "Devhelp",
+                               "name", _("Devhelp"),
                                "version", PACKAGE_VERSION,
                                "comments", _("A developer's help browser for GNOME"),
                                "authors", authors,
@@ -372,11 +443,29 @@ window_activate_about (GtkAction *action,
                                "translator-credits",
                                strcmp (translator_credits, "translator_credits") != 0 ?
                                translator_credits : NULL,
-                               "website", "http://developer.imendio.com/projects/devhelp/",
+                               "website", "http://live.gnome.org/devhelp",
                                "logo-icon-name", "devhelp",
                                NULL);
 }
 
+static void
+window_open_link_cb (DhWindow *window,
+                     const char *location,
+                     DhOpenLinkFlags flags)
+{
+        DhWindowPriv *priv;
+        priv = window->priv;
+
+        if (flags & DH_OPEN_LINK_NEW_TAB) {
+                window_open_new_tab (window, location, FALSE);
+        }
+        else if (flags & DH_OPEN_LINK_NEW_WINDOW) {
+                GtkWidget *new_window;
+                new_window = dh_base_new_window (priv->base);
+                gtk_widget_show (new_window);
+        }
+}
+
 static const GtkActionEntry actions[] = {
         { "FileMenu", NULL, N_("_File") },
         { "EditMenu", NULL, N_("_Edit") },
@@ -438,6 +527,11 @@ static const GtkActionEntry actions[] = {
           G_CALLBACK (window_activate_about) },
 };
 
+static const gchar* important_actions[] = {
+        "Back",
+        "Forward"
+};
+
 static void
 window_finalize (GObject *object)
 {
@@ -455,6 +549,18 @@ dh_window_class_init (DhWindowClass *klass)
 
         object_class->finalize = window_finalize;
 
+        signals[OPEN_LINK] =
+                g_signal_new ("open-link",
+                              G_TYPE_FROM_CLASS (klass),
+                              G_SIGNAL_RUN_LAST,
+                              G_STRUCT_OFFSET (DhWindowClass, open_link),
+                              NULL, NULL,
+                              _dh_marshal_VOID__STRING_FLAGS,
+                              G_TYPE_NONE,
+                              2,
+                              G_TYPE_STRING,
+                              DH_TYPE_OPEN_LINK_FLAGS);
+
         gtk_rc_parse_string ("style \"devhelp-tab-close-button-style\"\n"
                              "{\n"
                              "GtkWidget::focus-padding = 0\n"
@@ -480,13 +586,13 @@ dh_window_init (DhWindow *window)
         priv = GET_PRIVATE (window);
         window->priv = priv;
 
+        priv->selected_search_link = NULL;
+
         priv->manager = gtk_ui_manager_new ();
 
         accel_group = gtk_ui_manager_get_accel_group (priv->manager);
         gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
 
-        priv->zoom_level = 2;
-
         priv->main_box = gtk_vbox_new (FALSE, 0);
         gtk_widget_show (priv->main_box);
 
@@ -498,6 +604,11 @@ dh_window_init (DhWindow *window)
 
         gtk_container_add (GTK_CONTAINER (window), priv->main_box);
 
+        g_signal_connect (window,
+                          "open-link",
+                          G_CALLBACK (window_open_link_cb),
+                          window);
+
         g_signal_connect (priv->manager,
                           "add-widget",
                           G_CALLBACK (window_manager_add_widget),
@@ -513,6 +624,12 @@ dh_window_init (DhWindow *window)
                                       G_N_ELEMENTS (actions),
                                       window);
 
+        for (i = 0; i < G_N_ELEMENTS (important_actions); i++) {
+                action = gtk_action_group_get_action (priv->action_group,
+                                                      important_actions[i]);
+                g_object_set (action, "is-important", TRUE, NULL);
+        }
+
         gtk_ui_manager_insert_action_group (priv->manager,
                                             priv->action_group,
                                             0);
@@ -612,11 +729,22 @@ window_web_view_switch_page_cb (GtkNotebook     *notebook,
 
                 window_update_title (window, new_web_view, NULL);
         } else {
-                gtk_window_set_title (GTK_WINDOW (window), "Devhelp");
+                /* i18n: Please don't translate "Devhelp" (it's marked as translatable
+                 * for transliteration only) */
+                gtk_window_set_title (GTK_WINDOW (window), _("Devhelp"));
                 window_check_history (window, NULL);
         }
 }
 
+static void
+window_web_view_switch_page_after_cb (GtkNotebook     *notebook,
+                                      GtkNotebookPage *page,
+                                      guint            new_page_num,
+                                      DhWindow        *window)
+{
+        window_update_zoom_actions_sensitiveness (window);
+}
+
 static void
 window_populate (DhWindow *window)
 {
@@ -729,12 +857,18 @@ window_populate (DhWindow *window)
         priv->notebook = gtk_notebook_new ();
         gtk_container_set_border_width (GTK_CONTAINER (priv->notebook), 0);
         gtk_notebook_set_show_border (GTK_NOTEBOOK (priv->notebook), FALSE);
+        gtk_notebook_set_scrollable (GTK_NOTEBOOK (priv->notebook), TRUE);
         gtk_box_pack_start (GTK_BOX (priv->vbox), priv->notebook, TRUE, TRUE, 0);
 
         g_signal_connect (priv->notebook,
                           "switch-page",
                           G_CALLBACK (window_web_view_switch_page_cb),
                           window);
+        g_signal_connect_after (priv->notebook,
+                                "switch-page",
+                                G_CALLBACK (window_web_view_switch_page_after_cb),
+                                window);
+
 
         /* Create findbar. */
         priv->findbar = egg_find_bar_new ();
@@ -764,29 +898,47 @@ window_populate (DhWindow *window)
 
         gtk_widget_show_all (priv->hpaned);
 
-        window_open_new_tab (window, NULL);
+        window_update_zoom_actions_sensitiveness (window);
+        window_open_new_tab (window, NULL, TRUE);
 }
 
-static WebKitNavigationResponse
-window_web_view_navigation_requested_cb (WebKitWebView        *web_view,
-                                         WebKitWebFrame       *frame,
-                                         WebKitNetworkRequest *request,
-                                         DhWindow             *window)
+static gboolean
+window_web_view_navigation_policy_decision_requested (WebKitWebView             *web_view,
+                                                      WebKitWebFrame            *frame,
+                                                      WebKitNetworkRequest      *request,
+                                                      WebKitWebNavigationAction *navigation_action,
+                                                      WebKitWebPolicyDecision   *policy_decision,
+                                                      DhWindow                  *window)
 {
         DhWindowPriv *priv;
+        const char   *uri;
 
         priv = window->priv;
 
+        uri = webkit_network_request_get_uri (request);
+
+        if (webkit_web_navigation_action_get_button (navigation_action) == 2) { /* middle click */
+                webkit_web_policy_decision_ignore (policy_decision);
+                g_signal_emit (window, signals[OPEN_LINK], 0, uri, DH_OPEN_LINK_NEW_TAB);
+                return TRUE;
+        }
+
+        if (strncmp (uri, "file://", 7) != 0) {
+                webkit_web_policy_decision_ignore (policy_decision);
+                gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, NULL);
+                return TRUE;
+        }
+
+        if (strcmp (uri, "about:blank") == 0) {
+                return FALSE;
+        }
+
         if (web_view == window_get_active_web_view (window)) {
-                const gchar *uri;
-
-                uri = webkit_network_request_get_uri (request);
-
                 dh_book_tree_select_uri (DH_BOOK_TREE (priv->book_tree), uri);
                 window_check_history (window, web_view);
         }
 
-        return WEBKIT_NAVIGATION_RESPONSE_ACCEPT;
+        return FALSE;
 }
 
 static void
@@ -802,21 +954,10 @@ window_tree_link_selected_cb (GObject  *ignored,
 
         view = window_get_active_web_view (window);
 
-        /* Block so we don't try to sync the tree when we have already
-         * clicked in it.
-         */
-        g_signal_handlers_block_by_func (view,
-                                         window_web_view_navigation_requested_cb,
-                                         window);
-
         uri = dh_link_get_uri (link);
         webkit_web_view_open (view, uri);
         g_free (uri);
 
-        g_signal_handlers_unblock_by_func (view,
-                                           window_web_view_navigation_requested_cb,
-                                           window);
-
         window_check_history (window, view);
 }
 
@@ -831,6 +972,8 @@ window_search_link_selected_cb (GObject  *ignored,
 
         priv = window->priv;
 
+        priv->selected_search_link = link;
+
         view = window_get_active_web_view (window);
 
         uri = dh_link_get_uri (link);
@@ -881,9 +1024,11 @@ window_web_view_title_changed_cb (WebKitWebView  *web_view,
                                   const gchar    *title,
                                   DhWindow       *window)
 {
-        window_update_title (window,
-                             window_get_active_web_view (window),
-                             title);
+        if (web_view == window_get_active_web_view (window)) {
+                window_update_title (window, web_view, title);
+        }
+
+        window_tab_set_title (window, web_view, title);
 }
 
 static gboolean
@@ -1030,9 +1175,10 @@ window_web_view_tab_accel_cb (GtkAccelGroup   *accel_group,
         }
 }
 
-static void
+static int
 window_open_new_tab (DhWindow    *window,
-                     const gchar *location)
+                     const gchar *location,
+                     gboolean     switch_focus)
 {
         DhWindowPriv *priv;
         GtkWidget    *view;
@@ -1077,16 +1223,10 @@ window_open_new_tab (DhWindow    *window,
                           G_CALLBACK (window_web_view_button_press_event_cb),
                           window);
 
-        g_signal_connect (view, "navigation-requested",
-                          G_CALLBACK (window_web_view_navigation_requested_cb),
+        g_signal_connect (view, "navigation-policy-decision-requested",
+                          G_CALLBACK (window_web_view_navigation_policy_decision_requested),
                           window);
 
-        /*
-        g_signal_connect (view, "open-new-tab",
-                          G_CALLBACK (window_web_view_open_new_tab_cb),
-                          window);
-        */
-
         num = gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook),
                                         scrolled_window, NULL);
 
@@ -1100,12 +1240,16 @@ window_open_new_tab (DhWindow    *window,
         }
 
         if (location) {
-                webkit_web_view_open (WEBKIT_WEB_VIEW (view), location);
+                webkit_web_view_load_uri (WEBKIT_WEB_VIEW (view), location);
         } else {
                 webkit_web_view_open (WEBKIT_WEB_VIEW (view), "about:blank");
         }
 
-        gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), num);
+        if (switch_focus) {
+                gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook), num);
+        }
+
+        return num;
 }
 
 #ifndef GDK_WINDOWING_QUARTZ
@@ -1128,12 +1272,14 @@ tab_label_style_set_cb (GtkWidget *hbox,
         PangoFontMetrics *metrics;
         PangoContext     *context;
         GtkWidget        *button;
+        GtkStyle         *style;
         gint              char_width;
         gint              h, w;
 
         context = gtk_widget_get_pango_context (hbox);
+        style = gtk_widget_get_style (hbox);
         metrics = pango_context_get_metrics (context,
-                                             hbox->style->font_desc,
+                                             style->font_desc,
                                              pango_context_get_language (context));
 
         char_width = pango_font_metrics_get_approximate_digit_width (metrics);
@@ -1235,8 +1381,6 @@ window_update_title (DhWindow      *window,
                 web_view_title = webkit_web_frame_get_title (web_frame);
         }
 
-        window_tab_set_title (window, web_view, web_view_title);
-
         if (web_view_title && *web_view_title == '\0') {
                 web_view_title = NULL;
         }
@@ -1249,7 +1393,9 @@ window_update_title (DhWindow      *window,
         }
 
         if (!book_title) {
-                book_title = "Devhelp";
+                /* i18n: Please don't translate "Devhelp" (it's marked as translatable
+                 * for transliteration only) */
+                book_title = _("Devhelp");
         }
 
         if (web_view_title) {
